/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.empfang.dekoder;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3MeldungsDaten;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCStatus.TMCStatusEnum;

/**
 * Testcase f&uuml;r das Einlesen von TIC-Info Dateien.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class TestParser {

	/**
	 * Testmethode Einlesen Datei.
	 */
	@Test
	public void testParser1() {
		// URL dateiurl =
		// ClassLoader.getSystemResource("de/bsvrz/kex/lms/empfang/dekoder/TICMeldungen.xml");

		final String testdatei = "res/TIC3TestMeldung.XML";

		try {
			final TIC3InfoDatei datei = new TIC3InfoDatei(new File(testdatei));

			final TIC3InfoDateiDaten dateiMeldungen = datei.bildeMeldungen(false);
			final List<TIC3MeldungsDaten> meldungen = new ArrayList<>(dateiMeldungen.getMeldungen());
			Assert.assertEquals(1, meldungen.size());

			final TIC3MeldungsDaten meldung = meldungen.get(0);

			Assert.assertEquals("DatenKennung", "LMS-BW/r_LMS-BW/1266", meldung.getDatenKennung());
			Assert.assertEquals("DatenProduzent", "D BW LMS-BW", meldung.getDatenProduzent());
			Assert.assertEquals("Beschreibung",
					"A5 Karlsruhe Richtung Basel\nzwischen Dreieck Weil am Rhein und Grenzübergang Weil am Rhein/Basel\n1 km Stau für LKW",
					meldung.getEreignisBeschreibung());
			Assert.assertEquals("Status", TMCStatusEnum.Geaendert, meldung.getTMCMeldungsStatus());

		} catch (final Exception e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * Testmethode Einlesen Datei.
	 */
	@Test
	public void testParser2() {
		// URL dateiurl =
		// ClassLoader.getSystemResource("de/bsvrz/kex/lms/empfang/dekoder/TICMeldungen.xml");

		final String testdatei = "res/TIC3_2_ungueltige_Meldungen.XML";

		try {
			final TIC3InfoDatei datei = new TIC3InfoDatei(new File(testdatei));

			final TIC3InfoDateiDaten dateiMeldungen = datei.bildeMeldungen(false);
			final List<TIC3MeldungsDaten> meldungen = new ArrayList<>(dateiMeldungen.getMeldungen());
			Assert.assertEquals(2, meldungen.size());

			// final TIC3MeldungsDaten meldung = meldungen.get(0);
			//
			// Assert.assertEquals("DatenKennung", "LMS-BW/r_LMS-BW/1266",
			// meldung.getDatenKennung());
			// Assert.assertEquals("DatenProduzent", "D BW LMS-BW",
			// meldung.getDatenProduzent());
			// Assert.assertEquals("Beschreibung",
			// "A5 Karlsruhe Richtung Basel zwischen Dreieck Weil am Rhein und
			// Grenzübergang Weil am Rhein/Basel 1 km Stau für LKW",
			// meldung.getEreignisBeschreibung());
			// Assert.assertEquals("Status", TMCStatusEnum.Geaendert,
			// meldung.getTMCMeldungsStatus());

		} catch (final Exception e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * Testmethode Einlesen Datei.
	 */
	@Test
	public void testParser3() {
		// URL dateiurl =
		// ClassLoader.getSystemResource("de/bsvrz/kex/lms/empfang/dekoder/TICMeldungen.xml");

		final String testdatei = "res/TIC2MeldungTest1.xml";

		try {
			final TIC3InfoDatei datei = new TIC3InfoDatei(new File(testdatei));

			final TIC3InfoDateiDaten dateiMeldungen = datei.bildeMeldungen(false);
			final List<TIC3MeldungsDaten> meldungen = new ArrayList<>(dateiMeldungen.getMeldungen());
			Assert.assertEquals(0, meldungen.size());
		} catch (final Exception e) {
			Assert.fail(e.getMessage());
		}
	}

	private TIC3InfoElement findElement(final TIC3MeldungsDaten meldung, final String xmlElement) {

		for (TIC3InfoElement e : meldung.getKindElemente()) {
			if (e.getXmlElementName().equals(xmlElement)) {
				return e;
			}
		}

		return null;
	}
}
