/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Wei�enfelser Stra�e 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.empfang.dekoder;

import java.io.File;
import java.util.Collection;

import org.junit.Assert;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.kex.lmstic3.vew.TMCMeldung;
import de.bsvrz.kex.lmstic3.vew.TestDav;
import de.bsvrz.kex.lmstic3.vew.daten.ExterneMeldungsDaten;
import de.bsvrz.kex.lmstic3.vew.daten.TMCKonvertierungException;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Converter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Meldung;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3MeldungsDaten;

/**
 * Testcase f&uuml;r das Einlesen von TIC-Info Dateien.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class TestPublizieren {

	/**
	 * Datenverteiler-Verbindung.
	 */
	private TestDav dav = new TestDav();

	/**
	 * Testmethode Einlesen Datei.
	 */
	// @Test
	public void testGetDocument() {
		// URL dateiurl =
		// ClassLoader.getSystemResource("de/bsvrz/kex/lms/empfang/dekoder/TICMeldungen.xml");

		// String testdatei = "C:\\TICMeldungen.xml";
		final String testdatei = "res/TIC3TestMeldung.XML";

		Assert.assertTrue(dav.initDav());
		// final SystemObject lms =
		// dav.getDav().getDataModel().getObject("lmst.bw");

		try {
			final TIC3InfoDatei datei = new TIC3InfoDatei(new File(testdatei));

			// final MutableSet meldungsmenge = ((ConfigurationObject)
			// lms).getMutableSet("RDSMeldungen");
			//
			final DataModel model = dav.getDav().getDataModel();
			final ConfigurationArea area = dav.getDav().getLocalConfigurationAuthority().getConfigurationArea();
			final TIC3InfoDateiDaten dateiMeldungen = datei.bildeMeldungen(false);
			final Collection<TIC3MeldungsDaten> meldungen = dateiMeldungen.getMeldungen();
			Assert.assertEquals(1, meldungen.size());

			// DynamischeObjekte doob =
			// DynamischeObjekte.getInstanz(dav.getDav());

			for (final TIC3MeldungsDaten md : meldungen) {
				final String pid = md.bildePid();
				SystemObject so = model.getObject(pid);
				if (so == null) {
					System.out.println("neue Meldung " + pid + " wird angelegt");
					final SystemObjectType type = model.getType("typ.tmcTic3infoMeldung");
					so = area.createDynamicObject((DynamicObjectType) type, pid, pid);
				}

				final TIC3Meldung tic3meldung = new TIC3Meldung(so, md);
				tic3meldung.anmeldenDav(dav.getDav());
				tic3meldung.publiziereDav(dav.getDav(), null);

				publiziereTMCMeldung(model, area, md);

				System.out.println("§");
			}
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		try {
			Thread.sleep(30000);
		} catch (final InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	void publiziereTMCMeldung(final DataModel model, final ConfigurationArea area,
			final ExterneMeldungsDaten tic3meldung) {
		final String pid = tic3meldung.getID();
		SystemObject so = model.getObject(pid);
		if (so == null) {
			System.out.println("neue Meldung " + pid + " wird angelegt");
			final SystemObjectType type = model.getType("typ.tmcVerkehrsMeldung");
			try {
				so = area.createDynamicObject((DynamicObjectType) type, pid, pid);
			} catch (final ConfigurationChangeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// meldungsmenge.add(so);
		}

		final AttributeGroup attributeGroup = dav.getDav().getDataModel().getAttributeGroup(TMCMeldung.ATG);
		final Data createData = dav.getDav().createData(attributeGroup);

		final TIC3Converter conv = new TIC3Converter(tic3meldung);
		try {
			conv.erzeugeDAVDaten(createData);
			final TMCMeldung tmcmeldung = new TMCMeldung(so);
			tmcmeldung.anmeldenDavEmpfang(dav.getDav());
			tmcmeldung.aktualisieren(dav.getDav(), createData);
		} catch (final TMCKonvertierungException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TIC3Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
}
