/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.empfang.dekoder;

import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.junit.Assert;
import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

/**
 * Testcase f&uuml;r das Dekoder-Modul.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class ValidierungTest {
	/** TIC-Info Schema Location. */
	@SuppressWarnings("unused")
	private final String ticSchemaDatei = "/daten/workspace_TIC3/de.bsvrz.kex.lmstic3/res/TIC3_XML_Schema_Traffic_and_Travel_Event_4.26.66.1_(long).xsd";
	private final String ticDatei = "res/meldungen_vrz/TIC3Meldungen.XML";

	/**
	 * Erzeugt das Document zur Datei. (Falls eine Verifikation der Datei gegen
	 * das Schema einmal möglich ist, sollte dies hier erfolgen)
	 *
	 * @throws TIC3InfoDateiException
	 *             bei Fehlern
	 */
	// @Test
	public void testParse() throws Exception {
		Document doc = null;

		final DocumentBuilderFactory docfactory = DocumentBuilderFactory.newInstance();

		final SchemaFactory schfactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		// final URL schemaurl = ClassLoader.getSystemResource(ticSchemaDatei);

		final URL schemaurl = new URL("file://" + ticSchemaDatei);

		Assert.assertNotNull(schemaurl);

		try {
			// Validierung gegen Schema nicht möglich
			// Aussage Herr Kuske, Fa. GEWI
			final Schema ticschema = schfactory.newSchema(schemaurl);
			docfactory.setSchema(ticschema);
			docfactory.setValidating(true);

			// docfactory.setAttribute(XMLConstants.FEATURE_SECURE_PROCESSING,
			// false);

			// System.setProperty("jdk.xml.maxOccurLimit", "7000");
			System.setProperty("maxOccurLimit", "0");

			final DocumentBuilder parser = docfactory.newDocumentBuilder();

			doc = parser.parse(ticDatei);
			System.out.println("OK");
		} catch (final Exception e) {
			Assert.fail(e.getMessage());
		}
	}

	// @Test
	public void testParseSax() throws Exception {
		Document doc = null;

		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, false);

		final SchemaFactory schfactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		schfactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, false);
		// final URL schemaurl = ClassLoader.getSystemResource(ticSchemaDatei);

		final URL schemaurl = new URL("file://" + ticSchemaDatei);
		Assert.assertNotNull(schemaurl);

		try {
			// Validierung gegen Schema nicht möglich
			// Aussage Herr Kuske, Fa. GEWI
			final Schema ticschema = schfactory.newSchema(schemaurl);
			spf.setSchema(ticschema);
			spf.setValidating(true);

			SAXParser newSAXParser = spf.newSAXParser();
			XMLReader xmlReader = newSAXParser.getXMLReader();
			xmlReader.setErrorHandler(new ErrorHandler() {

				@Override
				public void warning(SAXParseException exception) throws SAXException {
					System.err.println(exception.getMessage());

				}

				@Override
				public void fatalError(SAXParseException exception) throws SAXException {
					// TODO Auto-generated method stub
					System.err.println(exception.getMessage());
				}

				@Override
				public void error(SAXParseException exception) throws SAXException {
					// TODO Auto-generated method stub
					System.err.println(exception.getMessage());
				}
			});
			xmlReader.parse(ticDatei);
			System.out.println("OK");
		} catch (final Exception e) {
			Assert.fail(e.getMessage());
		}
	}
}
