/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;

/**
 * TIC3-Koordinaten.
 * 
 * <p>
 * TIC3 XML Element: Coordinate (COR), TIC3 Info Typ 'CoordinateType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Koordinaten.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3Koordinaten extends AbstractTIC3InfoElement {
	/**
	 * Coordinate in WGS 84 format.
	 */
	class TIC3LocationKoordinate extends AbstractTIC3InfoElement {

		private static final String XML_ELEMENT_LOCATIONLOCCOORDINATESX = "TIC3KoordinateX";

		private static final String XML_ELEMENT_LOCATIONLOCCOORDINATESY = "TIC3KoordinateY";

		private static final String XML_ELEMENT_COORDINATE_LATITUDE = "Latitude";

		private static final String XML_ELEMENT_COORDINATE_LONGITUDE = "Longitude";

		/**
		 * Der Wert der Koordinate.
		 */
		private double koordinatenwert;

		/**
		 * Erzeugt ein neues Objekt.
		 *
		 * @param xmlName
		 *            Name des XML-ELementes
		 * @param davName
		 *            Name des DAV-Attributes
		 */
		TIC3LocationKoordinate(final String xmlName, final String davName) {
			super(xmlName, davName);
		}

		@Override
		public void getDavDaten(final Data davdata) throws TIC3Exception {
			koordinatenwert = davdata.getScaledValue(getDavAttributName()).doubleValue();
		}

		@Override
		public void getXmlDaten(final Node node) throws TIC3Exception {
			koordinatenwert = Double.parseDouble(node.getTextContent());
		}

		@Override
		public void setDavDaten(final Data davdata) throws TIC3Exception {
			davdata.getScaledValue(getDavAttributName()).set(koordinatenwert);
		}

		@Override
		public void setXmlDaten(final Element element) {
			element.setTextContent(String.valueOf((long) koordinatenwert));
		}

		public double getKoordinatenwert() {
			return koordinatenwert;
		}

		public void setKoordinatenwert(final double wert) {
			koordinatenwert = wert;
		}
	}

	/**
	 * TIC3 XML Element: Coordinate (COR).
	 */
	public static final String XML_ELEMENT_KOORDINATE = "Coordinate";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_KOORDINATE = "TIC3Koordinaten";

	private TIC3LocationKoordinate breite = new TIC3LocationKoordinate(
			TIC3LocationKoordinate.XML_ELEMENT_COORDINATE_LATITUDE,
			TIC3LocationKoordinate.XML_ELEMENT_LOCATIONLOCCOORDINATESY);
	private TIC3LocationKoordinate laenge = new TIC3LocationKoordinate(
			TIC3LocationKoordinate.XML_ELEMENT_COORDINATE_LONGITUDE,
			TIC3LocationKoordinate.XML_ELEMENT_LOCATIONLOCCOORDINATESX);

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3Koordinaten() {
		super(TIC3Koordinaten.XML_ELEMENT_KOORDINATE, "");
		addKindElement(breite);
		addKindElement(laenge);
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		setDavDaten(davdata);
		for (final TIC3InfoElement child : getKindElemente()) {
			child.publizieren(davdata, parameter);
		}
	}

	public double getLaenge() {
		return laenge.getKoordinatenwert();
	}

	public double getBreite() {
		return breite.getKoordinatenwert();
	}

	public void setLaenge(final double wert) {
		laenge.setKoordinatenwert(wert);
	}

	public void setBreite(final double wert) {
		breite.setKoordinatenwert(wert);
	}

}
