/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3OptionalerDoubleWert;

/**
 * TIC3 Verkehrs- und Reiseereignisattribut.
 * <p>
 * Entspricht TIC3-Typ 'TrafficAndTravelEventAttributeType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3TICEventVerkehrsUndReiseereignisAttribut.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3TICEventVerkehrsUndReiseereignisAttribut extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: TrafficAndTravelEventAttribute (TEA), A list of
	 * traffic and travel event attributes corresponding to the traffic and
	 * travel event code.
	 */
	public static final String XML_ELEMENT_TICEVENT_ATTRIBUTE = "TrafficAndTravelEventAttribute";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_TICEVENT_ATTRIBUTE = "VerkehrsUndReiseereignisAttribute";

	/**
	 * Beschreibung.
	 * <p>
	 * Beschreibung aus Wert des 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", "Beschreibung");

	/**
	 * Attribut.
	 * <p>
	 * TIC3 Info XML Element: Attribute (ATT), A reference to "Traffic And
	 * Travel Event Attribute".
	 * </p>
	 */
	private TIC3Text attribut = new TIC3Text("Attribute", "Attribut", "value");

	/**
	 * Wert.
	 * <p>
	 * TIC3 Info XML Element: Value (VAL), The value which is assigned to the
	 * traffic and travel event code attribute.
	 * </p>
	 */
	private TIC3OptionalerDoubleWert wert = new TIC3OptionalerDoubleWert("Value", "Wert");

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3TICEventVerkehrsUndReiseereignisAttribut() {
		super(XML_ELEMENT_TICEVENT_ATTRIBUTE, DAV_ATTR_TICEVENT_ATTRIBUTE);
		addAttribut(beschreibung);
		addKindElement(attribut);
		addKindElement(wert);
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		for (final TIC3InfoElement child : getKindElemente()) {
			child.publizieren(davdata, parameter);
		}

		for (final TIC3InfoElement child : getAttribute()) {
			child.publizieren(davdata, parameter);
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName() + ": Beschreibung='" + beschreibung.getXmlValue() + "', Attribut='"
				+ attribut.getText() + "', Wert='" + wert.getWert() + "'";
	}
}
