/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.text.ParseException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.XMLTools;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * Repr&auml;sentiert eine TIC3-Info Zeitangabe.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Zeitstempel.java 64047 2016-10-27 15:42:01Z gieseler $
 */

public class TIC3Zeitstempel extends AbstractTIC3InfoElement {

	private boolean hasData;

	/**
	 * Zeitstempel.
	 */
	private long zeitstempel = 1; // 0 ist nicht erlaubt

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Zeitstempel(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		zeitstempel = davdata.getTimeValue(getDavAttributName()).getMillis();
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		try {
			zeitstempel = XMLTools.getTICDateTime(node.getTextContent());
		} catch (final ParseException e) {
			throw new TIC3Exception("Fehler beim Einlesen des TIC3-Zeitstempels '" + node.getTextContent()
					+ "' für Element '" + getXmlElementName() + "'");
		}
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		setDavDaten(davdata, getDavAttributName());
	}

	@Override
	public void setDavDaten(final Data davdata, final String itemname) throws TIC3Exception {
		final Data item = davdata.getItem(itemname);
		if (item.isPlain()) {
			davdata.getTimeValue(itemname).setMillis(zeitstempel);
		}

		if (item.isArray()) {
			// optionale Zeitangabe
			final Array array = davdata.getArray(itemname);
			if (hasData) {
				array.setLength(1);
				array.getItem(0).asTimeValue().setMillis(zeitstempel);
			} else {
				array.setLength(0);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		if (hasData) {
			super.publizieren(davdata, parameter);
		}
	}

	@Override
	public void setXmlDaten(final Element element) {
		if (zeitstempel != 1) {
			element.setTextContent(XMLTools.getTICDateTime(zeitstempel));
		}
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);
		hasData = true;
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName() + " '" + getXmlElementName() + "'";
	}

	/**
	 * Gibt das Flag zur&uuml;ck, ob Daten vorhanden sind.
	 * 
	 * @return <code>true</code> wenn Daten vorhanden, sonst <code>false</code>
	 */
	public boolean hasData() {
		return hasData;
	}

	/**
	 * Setzt den Zeitstempel.
	 * 
	 * @param zeitstempel
	 *            Zeitstempel
	 */
	public void setZeitstempel(final Long zeitstempel) {
		if (zeitstempel != null) {
			this.zeitstempel = zeitstempel;
		}
	}
}
